'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _set2 = require('lodash/set');

var _set3 = _interopRequireDefault(_set2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _redux = require('redux');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var START = _constants.actionTypes.START,
    SET = _constants.actionTypes.SET,
    SET_ORDERED = _constants.actionTypes.SET_ORDERED,
    SET_PROFILE = _constants.actionTypes.SET_PROFILE,
    LOGIN = _constants.actionTypes.LOGIN,
    LOGOUT = _constants.actionTypes.LOGOUT,
    LOGIN_ERROR = _constants.actionTypes.LOGIN_ERROR,
    NO_VALUE = _constants.actionTypes.NO_VALUE,
    AUTHENTICATION_INIT_STARTED = _constants.actionTypes.AUTHENTICATION_INIT_STARTED,
    AUTHENTICATION_INIT_FINISHED = _constants.actionTypes.AUTHENTICATION_INIT_FINISHED,
    UNAUTHORIZED_ERROR = _constants.actionTypes.UNAUTHORIZED_ERROR,
    AUTH_UPDATE_SUCCESS = _constants.actionTypes.AUTH_UPDATE_SUCCESS;


var pathToArr = function pathToArr(path) {
  return path ? path.split(/\//).filter(function (p) {
    return !!p;
  }) : [];
};

/**
 * Reducer for requesting state. Changed by `START` and `SET` actions.
 * @param  {Object} state - Current requesting redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var requestingReducer = function requestingReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments[1];
  var path = action.path,
      requesting = action.requesting;

  switch (action.type) {
    case START:
    case SET:
      return _extends({}, state, _defineProperty({}, pathToArr(path).join('/'), requesting));
    // TODO: Handle NO_VALUE case
    // case NO_VALUE:
    default:
      return state;
  }
};

var getPathStr = function getPathStr(path) {
  return path ? path.replace('/', '.') : '';
};

/**
 * Reducer for data state. Changed by `LOGIN`, `LOGOUT`, and `LOGIN_ERROR`
 * actions.
 * @param  {Object} state - Current data redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var dataReducer = function dataReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments[1];
  var path = action.path,
      data = action.data,
      ordered = action.ordered;

  switch (action.type) {
    case SET:
      return _extends({}, state, (0, _set3.default)({}, getPathStr(path), data));
    case SET_ORDERED:
      return _extends({}, state, (0, _set3.default)({}, getPathStr(path), ordered));
    case NO_VALUE:
      return _extends({}, state, (0, _set3.default)({}, getPathStr(path), {}));
    default:
      return state;
  }
};

/**
 * Reducer for auth state. Changed by `LOGIN`, `LOGOUT`, and `LOGIN_ERROR`
 * actions.
 * @param  {Object} state - Current auth redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var authReducer = function authReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments[1];

  switch (action.type) {
    case LOGIN:
    case AUTH_UPDATE_SUCCESS:
      return action.auth || undefined;
    case LOGOUT:
    case LOGIN_ERROR:
      return null;
    default:
      return state;
  }
};

/**
 * Reducer for profile state. Changed by `SET_PROFILE`, `LOGOUT`, and
 * `LOGIN_ERROR` actions.
 * @param  {Object} state - Current profile redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var profileReducer = function profileReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var action = arguments[1];

  switch (action.type) {
    case SET_PROFILE:
      return _extends({}, state, action.profile);
    case LOGOUT:
    case LOGIN_ERROR:
      return null;
    default:
      return state;
  }
};

/**
 * Reducer for isInitializing state. Changed by `AUTHENTICATION_INIT_STARTED`
 * and `AUTHENTICATION_INIT_FINISHED` actions.
 * @param  {Object} state - Current isInitializing redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var isInitializingReducer = function isInitializingReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var action = arguments[1];

  switch (action.type) {
    case AUTHENTICATION_INIT_STARTED:
      return true;
    case AUTHENTICATION_INIT_FINISHED:
      return false;
    default:
      return state;
  }
};

/**
 * Reducer for errors state. Changed by `UNAUTHORIZED_ERROR`
 * and `LOGOUT` actions.
 * @param  {Object} state - Current authError redux state
 * @param  {object} action - Object containing the action that was dispatched
 * @return {Object} Profile state after reduction
 */
var errorsReducer = function errorsReducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var action = arguments[1];

  switch (action.type) {
    case UNAUTHORIZED_ERROR:
      return [].concat(_toConsumableArray(state), [action.payload]);
    case LOGOUT:
      return null;
    default:
      return state;
  }
};

/**
 * @name firebaseStateReducer
 * @description Reducer for react redux firebase. This function is called
 * automatically by redux every time an action is fired. Based on which action
 * is called and its payload, the reducer will update redux state with relevant
 * changes.
 * @param {Object} state - Current Redux State
 * @param {Object} action - Action which will modify state
 * @param {String} action.type - Type of Action being called
 * @param {String} action.data - Type of Action which will modify state
 * @return {Object} State
 */
exports.default = (0, _redux.combineReducers)({
  requesting: requestingReducer,
  data: dataReducer,
  auth: authReducer,
  profile: profileReducer,
  isInitializing: isInitializingReducer,
  errors: errorsReducer
});
module.exports = exports['default'];