'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.populate = exports.buildChildList = exports.isEmpty = exports.isLoaded = undefined;

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

var _compact2 = require('lodash/compact');

var _compact3 = _interopRequireDefault(_compact2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _defaultsDeep2 = require('lodash/defaultsDeep');

var _defaultsDeep3 = _interopRequireDefault(_defaultsDeep2);

var _reduce2 = require('lodash/reduce');

var _reduce3 = _interopRequireDefault(_reduce2);

var _every2 = require('lodash/every');

var _every3 = _interopRequireDefault(_every2);

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _split2 = require('lodash/split');

var _split3 = _interopRequireDefault(_split2);

var _last2 = require('lodash/last');

var _last3 = _interopRequireDefault(_last2);

var _has2 = require('lodash/has');

var _has3 = _interopRequireDefault(_has2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _set2 = require('lodash/set');

var _set3 = _interopRequireDefault(_set2);

var _size2 = require('lodash/size');

var _size3 = _interopRequireDefault(_size2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _populate = require('./utils/populate');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @description Detect whether items are loaded yet or not
 * @param {Object} item - Item to check loaded status of. A comma seperated list is also acceptable.
 * @return {Boolean} Whether or not item is loaded
 * @example
 * import React, { Component } from 'react'
 * import PropTypes from 'prop-types'
 * import { connect } from 'react-redux'
 * import { firebaseConnect, isLoaded } from 'react-redux-firebase'
 *
 * @firebaseConnect(['/todos'])
 * @connect(
 *   ({ firebase: { data: { todos } } }) => ({
 *     todos,
 *   })
 * )
 * class Todos extends Component {
 *   static propTypes = {
 *     todos: PropTypes.object
 *   }
 *
 *   render() {
 *     const { todos } = this.props;
 *
 *     // Show loading while todos are loading
 *     if(!isLoaded(todos)) {
 *        return <span>Loading...</span>
 *     }
 *
 *     return <ul>{todosList}</ul>
 *   }
 * }
 */
var isLoaded = exports.isLoaded = function isLoaded() {
  if (!arguments || !arguments.length) {
    return true;
  }

  return (0, _map3.default)(arguments, function (a) {
    return a !== undefined;
  }).reduce(function (a, b) {
    return a && b;
  });
};

/**
 * @description Detect whether items are empty or not
 * @param {Object} item - Item to check loaded status of. A comma seperated list is also acceptable.
 * @return {Boolean} Whether or not item is empty
 * @example
 * import React, { Component } from 'react'
 * import PropTypes from 'prop-types'
 * import { connect } from 'react-redux'
 * import { firebaseConnect, isEmpty } from 'react-redux-firebase'
 *
 * @firebaseConnect(['/todos'])
 * @connect(
 *   ({ firebase: { data: { todos } } }) => ({
 *     todos // state.firebase.data.todos from redux passed as todos prop
 *   })
 * )
 * class Todos extends Component {
 *   static propTypes = {
 *     todos: PropTypes.object
 *   }
 *
 *   render() {
 *     const { todos } = this.props;
 *
 *     // Message for if todos are empty
 *     if(isEmpty(todos)) {
 *        return <span>No Todos Found</span>
 *     }
 *
 *     return <ul>{todosList}</ul>
 *   }
 * }
 */
var isEmpty = exports.isEmpty = function isEmpty(data) {
  return !(data && (0, _size3.default)(data));
};

/**
 * @private
 * @description Build child list based on populate
 * @param {Object} data - Immutable Object to be converted to JS object (state.firebase)
 * @param {Object} list - Path of parameter to load
 * @param {Object} populate - Object with population settings
 */
var buildChildList = exports.buildChildList = function buildChildList(state, list, p) {
  return (0, _mapValues3.default)(list, function (val, key) {
    var getKey = val;
    // Handle key: true lists
    if (val === true) {
      getKey = key;
    }
    var pathString = p.childParam ? p.root + '.' + getKey + '.' + p.childParam : p.root + '.' + getKey;

    // console.log('path string:', { pathString, state })
    // Set to child under key if populate child exists
    if ((0, _get3.default)(state.data, pathString)) {
      return p.keyProp ? _extends(_defineProperty({}, p.keyProp, getKey), (0, _get3.default)(state.data, pathString)) : (0, _get3.default)(state.data, pathString);
    }
    // Populate child does not exist
    return val === true ? val : getKey;
  });
};

/**
 * @description Convert parameter under "data" path of Immutable Object to a
 * Javascript object with parameters populated based on populates array
 * @param {Object} firebase - Immutable Object to be converted to JS object (state.firebase)
 * @param {String} path - Path of parameter to load
 * @param {Array} populates - Array of populate objects
 * @param {Object|String|Boolean} notSetValue - Value to return if value is not found
 * @return {Object} Data located at path within Immutable Object
 * @example <caption>Basic</caption>
 * import { connect } from 'react-redux'
 * import { firebaseConnect } from 'react-redux-firebase'
 * const populates = [{ child: 'owner', root: 'users' }]
 *
 * const fbWrapped = firebaseConnect([
 *   { path: '/todos', populates } // load "todos" and matching "users" to redux
 * ])(App)
 *
 * export default connect((state) => ({
 *   // this.props.todos loaded from state.firebase.data.todos
 *   // each todo has child 'owner' populated from matching uid in 'users' root
 *   // for loading un-populated todos use state.firebase.data.todos
 *   todos: populate(state.firebase, 'todos', populates),
 * }))(fbWrapped)
 */
var populate = exports.populate = function populate(state, path, populates, notSetValue) {
  // TODO: Handle slash and lodash notation
  var pathArr = (0, _compact3.default)(path.split('/'));
  var dotPath = pathArr.join('.');
  // Handle undefined child
  if (!state || !(0, _get3.default)(state.data, dotPath)) {
    return notSetValue;
  }
  // test if data is a single object vs a list of objects, try generating
  // populates and testing for key presence
  var populatesForData = (0, _populate.getPopulateObjs)((0, _isFunction3.default)(populates) ? populates((0, _last3.default)((0, _split3.default)(path, '/')), (0, _get3.default)(state.data, dotPath)) : populates);
  var dataHasPopluateChilds = (0, _every3.default)(populatesForData, function (populate) {
    return (0, _has3.default)((0, _get3.default)(state.data, dotPath), populate.child);
  });
  if (dataHasPopluateChilds) {
    // Data is a single object, resolve populates directly
    return (0, _reduce3.default)((0, _map3.default)(populatesForData, function (p, obj) {
      // populate child is key
      if ((0, _isString3.default)((0, _get3.default)((0, _get3.default)(state.data, dotPath), p.child))) {
        var key = (0, _get3.default)((0, _get3.default)(state.data, dotPath), p.child);
        var pathString = p.childParam ? p.root + '.' + key + '.' + p.childParam : p.root + '.' + key;

        if ((0, _get3.default)(state.data, pathString)) {
          return (0, _set3.default)({}, p.child, p.keyProp ? _extends(_defineProperty({}, p.keyProp, key), (0, _get3.default)(state.data, pathString)) : (0, _get3.default)(state.data, pathString));
        }
        // matching child does not exist
        return (0, _get3.default)(state.data, dotPath);
      }
      return (0, _set3.default)({}, p.child, buildChildList(state, (0, _get3.default)((0, _get3.default)(state.data, dotPath), p.child), p));
    }),
    // combine data from all populates to one object starting with original data
    function (obj, v) {
      return (0, _defaultsDeep3.default)(v, obj);
    }, (0, _get3.default)(state.data, dotPath));
  } else {
    // Data is a map of objects, each value has parameters to be populated
    return (0, _mapValues3.default)((0, _get3.default)(state.data, dotPath), function (child, childKey) {
      var populatesForDataItem = (0, _populate.getPopulateObjs)((0, _isFunction3.default)(populates) ? populates(childKey, child) : populates);
      var resolvedPopulates = (0, _map3.default)(populatesForDataItem, function (p, obj) {
        // no matching child parameter
        if (!child || !(0, _get3.default)(child, p.child)) {
          return child;
        }
        // populate child is key
        if ((0, _isString3.default)((0, _get3.default)(child, p.child))) {
          var key = (0, _get3.default)(child, p.child);
          // attach child paramter if it exists
          var pathString = p.childParam ? p.root + '.' + key + '.' + p.childParam : p.root + '.' + key;
          if ((0, _get3.default)(state.data, pathString)) {
            return (0, _set3.default)({}, p.child, p.keyProp ? _extends(_defineProperty({}, p.keyProp, key), (0, _get3.default)(state.data, pathString)) : (0, _get3.default)(state.data, pathString));
          }
          // matching child does not exist
          return child;
        }
        // populate child list
        return (0, _set3.default)({}, p.child, buildChildList(state, (0, _get3.default)(child, p.child), p));
      });

      // combine data from all populates to one object starting with original data
      return (0, _reduce3.default)(resolvedPopulates, function (obj, v) {
        return (0, _defaultsDeep3.default)(v, obj);
      }, child);
    });
  }
};